<?php // doc_dir.php

class Album
{
  var $dirname;
  var $directories;
  var $imagefiles;

  function Album($dirname='.')
  {
    global $dirs;
    $this->dirname=$dirname;
    list($d,$i)=$dirs->read_contents($dirname);
    $this->directories=$d;
    $this->imagefiles=$i;
  }

  /** Utility function. Gets the first image from a directory.
   *  Recurses if necessary. Returns TRUE if $out_imagefiles is full. */
  function get_sample_imagefiles($max,&$out_samples)
  {
    // Find images.
    if(count($this->imagefiles)>0)
    {
      $this->imagefiles=array_values($this->imagefiles);
      for($i=0; $i<count($this->imagefiles); $i++)
      {
        $out_samples[] = $this->imagefiles[$i];
        if(count($out_samples) >= $max)
            return TRUE;
      }
    }
    // ...else recursively call into subdirs.
    if(count($this->directories)>0)
    {
      sort($this->directories);
      foreach($this->directories as $d)
      {
        $subalbum=new Album($this->dirname.'/'.$d);
        if( $subalbum->get_sample_imagefiles($max,$out_samples) )
            return TRUE;
      }
    }
    return FALSE;
  }

  /** Get the album thumbnail name. Make a new thumbnail if necessary. */
  function thumb()
  {
    global $dirs;
    $thumb_fname=$dirs->album($this->dirname).'/thumb.jpg';
    // If the thumbnail is up to date, just return it.
    if(!file_exists($thumb_fname))
    {
      // Generate the thumbnail.
      $samples=array();
      $this->get_sample_imagefiles(3,$samples);
      $sq=array();
      for($i=0; $i<3; $i++)
      {
        if($i < count($samples))
        {
          $ir=$samples[$i]->square();
          $sq[]=imagecreatefromjpeg( $ir->src );
        }
        else
        {
          $new=imagecreatetruecolor(75,75);
          $col=imagecolorallocate($new,240,240,240); // grey
          imagefill($new,0,0,$col);
          $sq[]=$new;
        }
      }
      $thumb=imagecreatetruecolor(113,75);
      $white=imagecolorallocate($thumb,255,255,255);
      imagefill($thumb,0,0,$white);
      //                 dst     src     dst-xy-src  dst-size-src
      imagecopyresampled($thumb, $sq[0], 0,0,   0,0, 75,75, 75,75 );
      imagecopyresampled($thumb, $sq[1], 76,0,  0,0, 37,37, 75,75 );
      imagecopyresampled($thumb, $sq[2], 76,38, 0,0, 37,37, 75,75 );
      // Save it.
      imagejpeg($thumb,$thumb_fname,85);
      imagedestroy($thumb);
      foreach($sq as $image)
          imagedestroy($image);
      chmod($thumb_fname,0666);
    }
    return $thumb_fname;
  }

} // end class Album

function table_dir_thumb($dir,$html,$is_here=false)
{
  $album=new Album($dir);
?>
  <table class="thumb"><tbody><tr>
  <td class="thumb" rowspan="2">
    <a href="<?php echo $dir
     ?>/?d=c"><img title="carousel" alt="carousel" width="113" height="75" src="<?php
         echo $album->thumb() ?>" /></a>
  </td><td class="detail">
    <?php echo $html; 
          if(count($album->imagefiles ))
            echo '<p>'.count($album->imagefiles ).' images</p>';
          if(!$is_here && count($album->directories))
            echo '<p>'.count($album->directories).' albums</p>'; ?>
  </td></tr><tr><td class="iconbox">
    <a class="on" href="<?php echo $dir
    ?>/?d=t"><img title="album" alt="album" src="<?php
      echo get_file(tsize_name().'.gif')?>" width="16" height="16" /></a>
  </td></tr></tbody></table>
<?php
  flush();
}

function doc_dir($directories)
{
  global $dirs;
  // Determine which page we should go to
  // [see the corresponding code in path_links()].
  if(array_key_exists('from_dir',$_GET))
    $referer_dir=array_search($_GET['from_dir'],$directories);
  else
    $referer_dir=0;
  $paged=new Paged(12,count($directories),$referer_dir);

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>

<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title><?php echo hsc($dirs->path()) ?></title>
<link rel="stylesheet" href="<?php echo get_file('exa.css') ?>" type="text/css" />
<?php load_theme() ?>
</head>

<body>

<div id="wrap" class="dir">

<div id="path"><?php echo $dirs->path_links(FALSE) ?></div>

<?php if(count($dirs->imagefiles)): ?>

<div id="header">
<?php table_dir_thumb('.','<h1>'.$dirs->current_album_title().'</h1>',true) ?>
</div>
<hr />

<?php endif; ?>

<div id="main">
<?php

for($i=$paged->begin(); $i<$paged->end(); $i++)
  table_dir_thumb($directories[$i],'<h3>'.nice($directories[$i]).'</h3>');

?>
</div> <!-- main -->

<?php $paged->index('?') ?>
<?php status() ?>

</div> <!-- wrap -->

</body>
</html>
<?php } // doc_dir() -----------------------------------------------------------

?>
